@@ -1,35 +1,39 @@
 /****************************************************************************
 **
 ** Copyright (C) 2017 Denis Shienkov <denis.shienkov@gmail.com>
+** Copyright (c) 2020 Andre Hartmann <aha_1980@gmx.de>
 ** Copyright (C) 2017 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtSerialBus module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL3$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
 ** packaging of this file. Please review the following information to
 ** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl.html.
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or later as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 2.0 requirements will be
-** met: http://www.gnu.org/licenses/gpl-2.0.html.
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -168,7 +172,7 @@
 #define PCAN_PCC                 0x06U  // PCAN-PC Card
 
 // PCAN parameters
-#define PCAN_DEVICE_NUMBER       0x01U  // PCAN-USB device number parameter
+#define PCAN_DEVICE_ID           0x01U  // Device identifier parameter
 #define PCAN_5VOLTS_POWER        0x02U  // PCAN-PC Card 5-Volt power parameter
 #define PCAN_RECEIVE_EVENT       0x03U  // PCAN receive event handler parameter
 #define PCAN_MESSAGE_FILTER      0x04U  // PCAN message filter parameter
@@ -197,6 +201,8 @@
 #define PCAN_BUSSPEED_DATA       0x1BU  // Configured CAN data speed as Bits per seconds
 #define PCAN_IP_ADDRESS          0x1CU  // Remote address of a LAN channel as string in IPv4 format
 #define PCAN_LAN_SERVICE_STATUS  0x1DU  // Status of the Virtual PCAN-Gateway Service
+#define PCAN_ATTACHED_CHANNELS_COUNT  0x2AU  // Get the amount of PCAN channels attached to a system
+#define PCAN_ATTACHED_CHANNELS        0x2BU  // Get information about PCAN channels attached to a system
 
 #define FEATURE_FD_CAPABLE       0x01U  // Device supports flexible data-rate (CAN-FD)
 
@@ -225,6 +231,10 @@
 #define TRACE_FILE_TIME          0x04U  // Includes the start time into the name of the trace file
 #define TRACE_FILE_OVERWRITE     0x80U  // Causes the overwriting of available traces (same name)
 
+// Other constants
+#define MAX_LENGTH_HARDWARE_NAME      33     // Maximum length of the name of a device: 32 characters + terminator
+#define MAX_LENGTH_VERSION_STRING     18     // Maximum length of a version string: 17 characters + terminator
+
 // PCAN message types
 #define PCAN_MESSAGE_STANDARD    0x00U  // The PCAN message is a CAN Standard Frame (11-bit identifier)
 #define PCAN_MESSAGE_RTR         0x01U  // The PCAN message is a CAN Remote-Transfer-Request Frame
@@ -269,11 +279,10 @@
 // Type definitions
 #ifdef Q_OS_MACOS
 #define TPCANLong                quint64
-#define TPCANLongToFrameID(a)    static_cast<quint32>(a)
 #else
 #define TPCANLong                quint32
-#define TPCANLongToFrameID(a)    a
 #endif
+#define TPCANLongToFrameID(a)    static_cast<QCanBusFrame::FrameId>(a)
 #define TPCANHandle              quint16   // Represents a PCAN hardware channel handle
 #define TPCANStatus              TPCANLong // Represents a PCAN status/error code
 #define TPCANParameter           quint8    // Represents a PCAN parameter to be read or set
@@ -312,6 +321,18 @@ typedef struct tagTPCANMsgFD
     quint8            DATA[64]; // Data of the message (DATA[0]..DATA[63])
 } TPCANMsgFD;
 
+// Describes an available PCAN channel
+typedef struct tagTPCANChannelInformation
+{
+    TPCANHandle channel_handle;                 // PCAN channel handle
+    TPCANDevice device_type;                    // Kind of PCAN device
+    quint8 controller_number;                   // CAN-Controller number
+    quint32 device_features;                    // Device capabilities flag (see FEATURE_*)
+    char device_name[MAX_LENGTH_HARDWARE_NAME]; // Device name
+    quint32 device_id;                          // Device number
+    quint32 channel_condition;                  // Availability status of a PCAN-Channel
+} TPCANChannelInformation;
+
 #define GENERATE_SYMBOL_VARIABLE(returnType, symbolName, ...) \
     typedef returnType (DRV_CALLBACK_TYPE *fp_##symbolName)(__VA_ARGS__); \
     static fp_##symbolName symbolName;
